<?php
/**
 * Plugin Messages (addon for plugin Adm_redacteurs)
 * Version  : 1.1 (2017/10/23)
 * Compatibility : Guppy v5.0+ et Adm_redacteurs v6.8+
 * Author   : jérôme CROUX (jchouix)
 * Web site : http://lebrikabrak.info/
 * E-mail   : lebrikabrak@free.fr
 * Licence  : GNU Lesser General Public License
 */

defined('ADMR_DIRNAME') OR exit('No direct script access allowed');

class Writer_messages_model
{       
    /*
    |--------------------------------------------------------------------------
    | Properties
    |--------------------------------------------------------------------------
    */
    private $_DB_file_path = ''; // Chemin du fichier contenant la base de données
    private $_file_dir_path = MESSAGES_DATA_PATH;    
    private $_file_prefix = MESSAGES_WRITER_FILE_PREFIX;
    private $_DB_counter_path = MESSAGES_NEXTID_DB; // Chemin du fichier contenant l'id du prochain document
    private $_data_DB = array();    //Données de la base de données
    private $_writer_id = 0; // Id du rédacteur
    private $_params = array(
        'id'     => 0, // Identifiant du document
        'author_id' => 0, // Id de l'auteur du document (0 = admin)
        'receiver_id' => 0, // Id du destinataire du document (0 = admin)
        'title' => '', // Titre du document (langue 1)
        'create_datetime' => '', // Date de création (timestamp)
        'mode' => 1 // 0 : lu, 1 : non lu 
    );

    /*
    |--------------------------------------------------------------------------
    | Constructor
    |--------------------------------------------------------------------------
    */

    /**
     * constructeur
     * 
     * @param array Tableau contenant l'identifiant du rédacteur qui est associé au fichier contenant la table des documents
     * @return void
     */ 
    function __construct($params = array())
    {      
        if (isset($params['id'])) {
            $this -> load_data($params['id']);
        }
    }
    
	// --------------------------------------------------------------------
 
	/**
	 * Charge les données des messages enregistrées dans le document du rédacteur
     * 
     * @access public 
	 * @param integer Identifiant du rédacteur
	 * @return array Tableau contenant la base de données des messages enregistrées dans le document du rédacteur (vide par défaut)
	 */    
    function load_data($writer_id)
    {
        if (isset($writer_id) && ctype_digit((string) $writer_id)) {
            $this -> _DB_file_path = $this -> get_DB_file_path($writer_id);
            $this -> _writer_id = $writer_id;
            $this -> _data_DB = $this -> read();   
        }
        
        return $this -> _data_DB;       
    }
    
	// --------------------------------------------------------------------
 
	/**
	 * Définit et renvoie le chemin du fichier contenant la base de donnée du rédacteur
     * 
	 * @access public
     * @param  integer Identifiant du rédacteur
	 * @return string Chemin du fichier contenant la base de donnée du rédacteur
	 */
    function set_DB_file_path($writer_id)
    {
        if (isset($writer_id) && ctype_digit((string) $writer_id)) {
            $this -> _DB_file_path = $this -> _file_dir_path.'/'.$this -> _file_prefix.$writer_id.DBEXT;
        }

        return $this -> _DB_file_path;
    }
     
	// --------------------------------------------------------------------
 
	/**
	 * renvoie le chemin du fichier contenant la base de donnée du rédacteur
     * 
	 * @access public
     * @param  integer Identifiant du rédacteur (optionnel)
	 * @return string Chemin du fichier contenant la base de donnée du rédacteur
	 */
    function get_DB_file_path($writer_id = 0)
    {
        return $this -> set_DB_file_path($writer_id);
    }
    
	// --------------------------------------------------------------------
 
	/**
	 * récupére les données des rédacteurs enregistrés (tous par défaut)
     * 
	 * @access public
     * @param  array Tableau de filtres (optionnel) sous la forme array($key1 => mixed $values1, $key2 => mixed $values2, ...)
     * @param string Option de tri (optionnel)
	 * @return array tableau contenant les données des rédacteurs enregistrés
	 */
	function get($filters = array(), $sort_by = '')
	{
        settype($filters, 'array');
        $data = array();
      
        if (!empty($filters) ) {
            $data = $this -> get_many_by_keys($filters);
        }
        else {
            $data = $this -> get_all();
        }
                
        if (!empty($sort_by)) {
            $data = $this -> sort_by_key($data, $sort_by);
        }
        
        return $data;
	}

	// --------------------------------------------------------------------

    /**
     * récupére tous les documents enregistrés
     * 
     * @access public
     * @return array
     */
    function get_all()
    {  
        return $this -> _data_DB;
    }

	// --------------------------------------------------------------------

    /**
     * récupérer le nombre de documents enregistrés
     * 
     * @access public
     * @param  string Type du document (optionnel)
     * @return integer
     */
	function count($filters = array('type' => NULL, 'author' => NULL))
	{
        if (isset($filters['type'])) {
            return $this -> count_by_type($type);
        }
        else if (isset($filters['author'])) {
            return $this -> count_by_author($author);
        }
        else {
            return $this -> count_all();
        }
 	}

	// --------------------------------------------------------------------

    /**
     * récupérer le nombre total de documents enregistrés
     * 
     * @access public
     * @return integer
     */
    function count_all()
    {
        return count($this -> _data_DB);
    }

	// --------------------------------------------------------------------

    /**
     * récupérer le nombre total de documents enregistrés par un auteur
     * 
     * @access public
     * @param  string Nom de l'auteur des documents
     * @return integer
     */
	function count_by_author($author)
	{
        return count($this -> get_many_by_author($author));
 	}

	// --------------------------------------------------------------------

    /**
     * récupérer les données identifié par la clé et sa valeur
     * 
     * @param array Données de la base
     * @param string Clé
     * @param string Valeur de la clé
     * @return array
     */
    function select_by_key($data_DB = array(), $key = '', $value = '')
    {
        settype($data_DB, 'array');
        settype($value, 'string');
        
        foreach($data_DB as $row) {
            if (isset($row[$key]) && $row[$key] == $value) {
                return $row;
            }
        }    
        
        return array();
    }

	// --------------------------------------------------------------------

    /**
     * récupérer les données identifié par la clé et sa ou ses valeurs
     * 
     * @param array Données de la base
     * @param string Clé
     * @param string|array Valeur(s) de la clé
     * @return array
     */
    function select_many_by_key($data_DB = array(), $key = '', $value = array())
    {
        settype($data_DB, 'array');
        settype($value, 'array');
        $data = array();
        
        foreach($data_DB as $row) {
            if (isset($row[$key]) && in_array($row[$key], $value)) {
                $data[$row['id']] = $row;
            }
        }    
        
        return $data;
    }

	// --------------------------------------------------------------------

    /**
     * récupérer les données identifié par la clé et sa ou ses valeurs
     * 
     * @param array Données de la base
     * @param array Tableau contenant les critères de sélection sous la forme array($key1 => mixed $values1, $key2 => mixed $values2,...)
     * @return array
     */
    function select_many_by_many_keys($data_DB = array(), $keys = array())
    {
        settype($data_DB, 'array');
        settype($keys, 'array');

        
        $data = $data_DB;

        foreach($keys as $key => $values) {
            
            if (empty($values)) {
                continue;    
            }
            
            settype($values, 'array');
            
            foreach($data as $id => $row) {
                if (!isset($row[$key]) || !in_array($row[$key], $values)) {
                    unset($data[$id]);
                }
            }           
        }        
        
        return $data;
    }

	// --------------------------------------------------------------------

    /**
     * récupérer les données du document identifié par son identifiant
     * 
     * @param integer|array Id du ou des rédacteur(s))
     * @return array Données du rédacteur retourné sous la forme array('name' => , 'id' => , 'status' => , 'groupid' => )
     */
    function get_by_id($id)
    {
        return $this -> select_by_key($this -> _data_DB, 'id', $id);
    }

	// --------------------------------------------------------------------

    /**
     * récupérer les données des document(s) identifié(s) par leurs identifiants
     * 
     * @param integer|array Id du ou des rédacteur(s))
     * @return array Données des documents retourné sous la forme
     *                     array( 
     *                          $id => array('name' => , 'id' => , 'status' => , 'groupid' => ),
     *                          $id => array('name' => , 'id' => , 'status' => , 'groupid' => ), 
     *                      )
     */
    function get_many_by_id($ids)
    {
        return $this -> select_many_by_key($this -> _data_DB, 'id', $ids);
    }

	// --------------------------------------------------------------------

    /**
     * récupérer les données des document(s) identifié(s) par leur type
     * 
     * @param string|array type du ou des documents(s))
     * @return array Données des documents retourné sous la forme
     *                     array( 
     *                          $id => array('id' => , 'status' => , 'type' =>  , 'title' => '', 'comment' => '', 'author' => ''),
     *                          $id => array('id' => , 'status' => , 'type' =>  , 'title' => '', 'comment' => '', 'author' => ''),
     *                      )
     */
    function get_many_by_type($types)
    {
        return $this -> select_many_by_key($this -> _data_DB, 'type', $types);
    }

	// --------------------------------------------------------------------

    /**
     * récupérer les données des documents identifiés par le status
     * 
     * @param string Status du rédacteur
     * @return array
     */
    function get_many_by_status($status)
    {
        return $this -> select_many_by_key($this -> _data_DB, 'status', $status);
    }

	// --------------------------------------------------------------------

    /**
     * récupérer les données des documents identifiés par le status
     * 
     * @param string Nom de l'auteur
     * @return array
     */
    function get_many_by_author($author)
    {
        return $this -> select_many_by_key($this -> _data_DB, 'author', $author);
    }

	// --------------------------------------------------------------------

    /**
     * récupérer les données des documents identifiés par les critères de sélection
     * 
     * @param array Tableau contenant les critères de sélection sous la forme array( key => value)
     * @return array
     */
    function get_many_by_keys($keys)
    {
        settype($keys, 'array');

        return $this -> select_many_by_many_keys($this -> _data_DB, $keys);
    }

	// --------------------------------------------------------------------

    /**
     * récupérer les données du document identifié par le status différent du sien
     * 
     * @param string Status différent de celui du document
     * @return array
     */
    function get_many_by_not_status($status)
    {
        return array_diff_key($this -> _data_DB, $this -> select_many_by_key($this -> _data_DB, 'status', $status));
    }

	// --------------------------------------------------------------------

    /**
     * récupére les données brutes de la table indexée des documents (id||status||type||title||||author||mode)
     *
     * @return array
     */
    private function read()
    {
        $data_DB = array();
        $records = array();
        
        if (is_file($this -> _DB_file_path)) {
            $data_DB = file($this -> _DB_file_path);
    		
            foreach($data_DB as $data) {
                list($id, $from, $to, $title, $time, $is_read) = explode(CONNECTOR, trim($data));
    			
                $records[$id] = array(
                    'id' => $id,      // Identifiant du document
                    'author_id' => $from, // Id de l'auteur du document (0 = admin)
                    'receiver_id' => $to, // Id du destinataire du document (0 = admin)
                    'title' => $title,   // Titre du document (lang 1)
                    'create_datetime' => $time, // Date de création (timestamp)
                    'mode' => $is_read // 0 : lu, 1 : non lu 
                );
            }
        }
        
        return $records;
    }

	// --------------------------------------------------------------------

    /**
     * Efface le contenu de la table et le remplace avec les données passées en paramètre
     * 
     * @param array Données à écrire
     * @return array
     */
    private function write()
    {   
        $data = $this -> _data_DB;
        $file_path = $this -> _DB_file_path;

        if (!empty($file_path)) {
            $data_out = '';
              
            foreach($data as $row) {     
                $data_out .= implode(CONNECTOR, array_values($row))."\n";
            }
    
            $fhandle = fopen($file_path, 'w');
            fputs($fhandle, $data_out);
            fclose($fhandle);
            
            return TRUE;
        }
        
        return FALSE;       
    }

	// --------------------------------------------------------------------

    /**
     * Ajoute un enregistrement dans la table indexée des documents (id||status||type||title||comment||author)
     * 
     * @param array Données de l'enregistrement du rédacteur dans la table
     * @return integer Id du compteur
     */
    function insert($data)
    {
        if (empty($data)) {
            return FALSE;
        }
        
        $data['id'] = $this -> update_counter_id();  // Mise à jour du compteur id

        // Insertion pour l'auteur
/*        $this -> _data_DB[$data['id']] = array_merge($this -> _params, $this -> _clean($data), array('id' => $data['id']));        
        $this -> write();*/
        $this -> _insert_for_author($data);
        
        // Insertion pour le destinataire
/*        $this -> _data_DB[$data['id']] = array_merge($this -> _params, $this -> _clean($data), array('id' => $data['id']));        
        $this -> write();*/
        $this -> _insert_for_receiver($data);       

        return $data['id'];
    }

	// --------------------------------------------------------------------

    /**
     * Ajoute un enregistrement dans la table indexée des documents de l'auteur du message
     * 
     * @param array Données de l'enregistrement de l'auteur du message dans la table
     * @return integer Id du compteur
     */
    private function _insert_for_author($data)
    {
        $this -> _data_DB[$data['id']] = array_merge($this -> _params, $this -> _clean($data), array('id' => $data['id']));        
        
        $this -> write();        
    }

	// --------------------------------------------------------------------

    /**
     * Ajoute un enregistrement dans la table indexée des documents de l'auteur du message
     * 
     * @param array Données de l'enregistrement de l'auteur du message dans la table
     * @return integer Id du compteur
     */
    private function _insert_for_receiver($data)
    {
        $this -> load_data($data['receiver_id']);
        
        $this -> _data_DB[$data['id']] = array_merge($this -> _params, $this -> _clean($data), array('id' => $data['id']));        
        
        $this -> write();        
    }


	// --------------------------------------------------------------------

    /**
     * Met à jour l'enregistrement identifié par le paramètre id 
     * dans la table indexée des documents (id||status||type||title||comment||author)
     * 
     * @param array Données de l'enregistrement du document à mettre à jours dans la table
     * @return array
     */
    function update($data)
    {
        return TRUE;
    }

	// --------------------------------------------------------------------

    /**
     * supprime les documents du rédacteur identifiés par leurs identifiants dans la table
     * 
     * @param integer|array Identifiants des documents dans la table
     * @return array
     */
    function delete_by_id($ids)
    {
        settype($ids, 'array');
        
        if (empty($ids)) {
            return FALSE;
        }
        
        $db =& $this -> _data_DB;
        
        foreach($db as $key => $row){
            if (in_array($row['id'], $ids)) {
                unset($db[$key]);
            }            
        }
            
        $this -> write();
        
        return TRUE;
    }

	// --------------------------------------------------------------------

    /**
     * supprime les messages du rédacteur
     * 
     * @param integer Identifiant du rédacteur (optionnel)
     * @return array
     */
    function destroy_writer_DB_file($writer_id = 0)
    {
        $this -> set_DB_file_path($writer_id);
        
        if (is_file($this -> _DB_file_path)) {
            @unlink($this -> _DB_file_path); // Suppression du fichier contenant la table de donnée du rédacteur           
        }
    }

	// --------------------------------------------------------------------
 
	/**
	 * Nettoie les données du rédacteur
     * 
     * @access private 
	 * @param array Données du rédacteur à nettoyer
	 * @return array
	 */        
    private function _clean($data)
    {
        settype($data, 'array');
        
        return array_map('RemoveConnector', array_intersect_key($data, $this -> _params));
    }

	// --------------------------------------------------------------------

    /**
     * Met à jour l'enregistrement identifié par le paramètre id 
     * dans la table indexée des documents (id||status||type||title||comment||author)
     * 
     * @param integer Id du document dans la table
     * @param array Données de l'enregistrement du document dans la table
     * @return array
     */
/*    function update_by_id($id, $data)
    {
        if (isset($this -> _data_DB[$id]) && !empty($data)) {  
            $this -> _data_DB[$id] = array(
                'id'      => $id,      // Identifiant du document
                'status'  => $data['status'],  // Status du document
                'type'    => $data['type'],    // Type du document
                'title'   => $data['title'],   // Titre du document
                'comment' => $data['comment'], // commentaire
                'author'  => $data['author'],  // Auteur 
            );            
            
            $this -> write();
            
            return TRUE;
        }
        
        return FALSE;
    }
*/
	// --------------------------------------------------------------------

    /**
     * Renvoie le compteur de l'id des enregistrements
     *
     * @return integer Identifiant du compteur
     */
    function get_counter_id()
    {
        $counter = $this -> _DB_counter_path;
        
        $id = ReadCounter($counter);
        
        return $id;
    }

	// --------------------------------------------------------------------

    /**
     * Renvoie le compteur de l'id des enregistrements
     *
     * @return integer Identifiant du compteur
     */
    function write_counter_id($id)
    {
        $counter = $this -> _DB_counter_path;
        
        WriteCounter($counter, $id);
        
        return $id;

    }

	// --------------------------------------------------------------------

    /**
     * Met à jour le compteur de l'id des enregistrements et renvoie le nouvel identifiant
     *
     * @return integer Identifiant du compteur
     */
    function update_counter_id()
    {
        $counter = $this -> _DB_counter_path;
        
        $id = ReadCounter($counter);
        $id++;
        WriteCounter($counter, $id);
        
        return $id;
    }

	// --------------------------------------------------------------------

    /**
     * Renvoie l'id enregistré dans le compteur
     *
     * @return integer id
     */
    function get_last_insert_id()
    {
        $counter = $this -> _DB_counter_path;
        
        return ReadCounter($counter);
    }

	// --------------------------------------------------------------------

    /**
     * Tri les enregistrements
     * 
     * @param string Option de tri
     * @return array Données triées
     */
    function sort_by_key($data, $sort_option)
    {       
        if(empty($data)) {
            return $data;
        }

        // Tableau en colonne
        foreach ($data as $key => $row) {
            $id[$key]     = $row['id'];
//            $title[$key]   = $row['title'];
            $from[$key] = $row['author_id'];
            $to[$key]   = $row['receiver_id'];
            $time[$key] = $row['create_datetime'];
            $mode[$key] = $row['mode'];
        }
        //
        
        switch ($sort_option) {
			case 'id_desc' :
				array_multisort($id, SORT_NUMERIC, SORT_DESC, $data);
				break;
			case 'id_asc' :
				array_multisort($id, SORT_NUMERIC, SORT_ASC, $data);
				break;
			case 'from_asc' :
				array_multisort($from, SORT_NATURAL, SORT_ASC, $data);
				break;
			case 'from_desc' :
				array_multisort($from, SORT_NATURAL, SORT_DESC, $data);
				break;
			case 'to_asc' :
				array_multisort($to, SORT_NATURAL, SORT_ASC, $data);
				break;
			case 'to_desc' :
				array_multisort($to, SORT_NATURAL, SORT_DESC, $data);
				break;
			case 'time_asc' :
				array_multisort($time, SORT_NUMERIC, SORT_ASC, $data);
				break;
			case 'time_desc' :
				array_multisort($time, SORT_NUMERIC, SORT_DESC, $data);
				break;
			case 'mode_asc' :
				array_multisort($mode, SORT_NUMERIC, SORT_ASC, $data);
				break;
			case 'mode_desc' :
				array_multisort($mode, SORT_NUMERIC, SORT_DESC, $data);
				break;
			default :
				array_multisort($time, SORT_NUMERIC, SORT_DESC, $data);
		}	
	
		return $data;
    }

	// --------------------------------------------------------------------

    /**
     * Met à jour les status des enregistrement dans la table 
     * 
     * @param array Id des enregistrements dans la table
     * @param string Status à affecter
     * @return array
     */
/*    function update_status_by_id($ids, $status)
    {
        settype($ids, 'array');
        $db =& $this -> _data_DB;
                
        foreach($db as $key => $row){
            if (in_array($row['id'], $ids)) {
                $db[$key]['status'] = $status;   
            }            
        }
            
        $this -> write();
        
        return TRUE;
    }*/

	// --------------------------------------------------------------------

    /**
     * Met à jour les status des enregistrement dans la table 
     * 
     * @param array type des documents
     * @param string Status à affecter
     * @return array
     */
/*    function update_status_by_type($types, $status)
    {
        settype($types, 'array');
        $db =& $this -> _data_DB;
                
        foreach($db as $key => $row){
            if (in_array($row['type'], $types)) {
                $db[$key]['status'] = $status;   
            }            
        }
            
        $this -> write();
        
        return TRUE;
    }*/

	// --------------------------------------------------------------------

    /**
     * Met à jour le mode des documents dans la table 
     * 
     * @param array Id des enregistrements dans la table
     * @param string Status à affecter
     * @return array
     */
    function update_mode_by_id($ids, $mode)
    {
        settype($ids, 'array');
        $db =& $this -> _data_DB;
                
        foreach($db as $key => $row){
            if (in_array($row['id'], $ids)) {
                $db[$key]['mode'] = $mode;   
            }            
        }
            
        $this -> write();
        
        return TRUE;
    }

	// --------------------------------------------------------------------
 
	/**
	 * Définit et renvoie l'intitulé du document
     * 
     * @access public 
	 * @param array Tableau contenant les paramètres du document
	 * @return string Intitulé descriptif du document
	 */
    public function set_doc_heading($params)
    {        
        $title = strip_tags($params['title']);
        
        if (strlen($title) > 142) {
            $title = CutLongWord($title, 142).' .../...';
        }
        
        return $title; 
    }

	// --------------------------------------------------------------------

} // End of Class